#!/usr/bin/python2.7
#
# Univention Management Console
#  Configuration of Zimbra
#
# Copyright 2016-2017 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import subprocess
import ldap.filter

from univention.management.console.base import Base, UMC_Error
from univention.management.console.modules.decorators import simple_response, sanitize
from univention.management.console.modules.sanitizers import StringSanitizer
from univention.management.console.ldap import get_user_connection
from univention.lib.i18n import Translation
from zimbra4ucs import ZimbraConfiguration
from zimbra4ucs.models import ZimbraMailDomains
from zimbra4ucs.errors import ZimbraError, ZimbraResponseEmpty
from univention.admin import modules

_ = Translation('zimbra4ucs').translate


class Instance(Base):

	@simple_response
	def query(self):
		return {
			'initialized': ZimbraConfiguration.is_initialized(),
		}

	def error_handling(self, etype, exc, etraceback):
		if isinstance(exc, ZimbraError):
			raise UMC_Error(str(exc), status=500)

	@sanitize(
		username=StringSanitizer(required=True),
		password=StringSanitizer(required=True),
		hostname=StringSanitizer(required=True),
	)
	@simple_response
	def get_mail_domains(self, hostname, username, password):
		zimbra = ZimbraConfiguration(hostname, username, password)
		zimbra.authenticate()

		try:
			domains = ZimbraMailDomains(zimbra.connection).get_all()
		except ZimbraResponseEmpty:
			raise ZimbraError(_('There are no mail domains configured, yet. Please configure a mail domain.'))

		return [
			dict(id=domain['name'], label=domain['name'])
			for domain in domains
		]

	@sanitize(
		username=StringSanitizer(required=True),
		password=StringSanitizer(required=True),
		hostname=StringSanitizer(required=True),
		maildomain=StringSanitizer(required=True),
	)
	@simple_response
	def configure(self, hostname, username, password, maildomain):
		zimbra = ZimbraConfiguration(hostname, username, password)
		zimbra.configure(maildomain)
		subprocess.call(['service', 'zimbra4ucs', 'restart'])
		self.create_ucs_maildomain(maildomain)

	def create_ucs_maildomain(self, name):
		def bind(lo):
			lo.bind(self.user_dn, self.password)
		lo, po = get_user_connection(bind=bind, write=True)
		co = None
		modules.update()
		md = modules.get('mail/domain')
		po.setDn('cn=domain,cn=mail,%s' % po.getBase())
		modules.init(lo, po, md)
		if not md.lookup(co, lo, ldap.filter.filter_format('cn=%s', [name])):
			md_obj = md.object(co, lo, po)
			md_obj.open()
			md_obj['name'] = name
			md_obj.create()
